@@ -4,6 +4,7 @@
  * Copyright (C) 2002-2003 Richard Hult <richard@imendio.com>
  * Copyright (C) 2002-2003 Mikael Hallendal <micke@imendio.com>
  * Copyright (C) 2002-2003 CodeFactory AB
+ * Copyright (C) 2003      Kevin Dougherty <gossip@kdough.net>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -44,9 +45,10 @@
 #include "gossip-join-dialog.h"
 #include "gossip-message.h"
 #include "gossip-sound.h"
-#include "gossip-app.h"
-
+#include "gossip-idle.h"
 #include "gossip-startup-druid.h"
+#include "gossip-preferences.h"
+#include "gossip-app.h"
 
 #define DEFAULT_RESOURCE "Gossip"
 #define ADD_CONTACT_RESPONSE_ADD 1
@@ -75,10 +77,13 @@ struct _GossipAppPriv {
 
 	/* The status to set when we get a connection. */
 	GossipStatus         status_to_set_on_connect;
+
+	gboolean             is_auto_away;
 	
 	/* Dialogs that we only have one of at a time. */
 	GossipJoinDialog    *join_dialog;
 	GtkWidget           *about;
+	GtkWidget           *preferences_dialog;
 
 	/* Widges that are enabled when we're connected. */
 	GList               *enabled_connected_widgets;
@@ -126,6 +131,7 @@ app_get_chat_for_jid                                (GossipApp      *app,
 static void      app_user_activated_cb              (GossipRoster   *roster,
 						     GossipJID      *jid,
 						     GossipApp      *app);
+static gboolean  app_idle_check_cb                  (GossipApp      *app);
 static void      app_set_status                     (GossipApp      *app,
 						     GossipStatus    status);
 static void      app_quit_cb                        (GtkWidget      *window,
@@ -134,6 +140,8 @@ static void      app_send_chat_message_cb           (GtkWidget      *widget,
 						     GossipApp      *app);
 static void      app_add_contact_cb                 (GtkWidget      *widget,
 						     GossipApp      *app);
+static void      app_preferences_cb                 (GtkWidget      *widget,
+						     GossipApp      *app);
 static void      app_about_cb                       (GtkWidget      *widget,
 						     GossipApp      *app);
 static void      app_join_group_chat_cb             (GtkWidget      *widget,
@@ -300,6 +308,7 @@ app_init (GossipApp *app)
 			      "actions_join_group_chat", "activate", app_join_group_chat_cb,
 			      "actions_send_chat_message", "activate", app_send_chat_message_cb,
 			      "actions_add_contact", "activate", app_add_contact_cb,
+			      "edit_preferences", "activate", app_preferences_cb,
 			      "help_about", "activate", app_about_cb,
 			      "main_window", "destroy", app_main_window_destroy_cb,
 			      NULL);
@@ -364,6 +373,9 @@ app_init (GossipApp *app)
 	app_set_status_indicator (app, GOSSIP_STATUS_OFFLINE);
 
 	app_update_connection_dependent_menu_items (app);
+
+	/* Start the idle time checker. */
+	g_timeout_add (5000, (GSourceFunc) app_idle_check_cb, app);
 	
 	priv->about = NULL;
 	gtk_widget_show (priv->window);
@@ -669,7 +681,24 @@ static void
 app_add_contact_cb (GtkWidget *widget, GossipApp *app)
 {
 	gossip_add_contact_new (app->priv->connection, NULL);
-	return;
+}
+
+static void
+app_preferences_cb (GtkWidget *widget, GossipApp *app)
+{
+	GossipAppPriv *priv;
+
+	priv = app->priv;
+
+	if (priv->preferences_dialog) {
+		gtk_window_present (GTK_WINDOW (priv->preferences_dialog));
+		return;
+	}
+
+	priv->preferences_dialog = gossip_preferences_show (app);
+
+	g_object_add_weak_pointer (G_OBJECT (priv->preferences_dialog),
+				   (gpointer) &priv->preferences_dialog);
 }
 
 static void
@@ -1022,7 +1051,6 @@ app_iq_handler (LmMessageHandler *handler,
 			LmMessage      *v;
 			const gchar    *from, *id;
 			struct utsname  osinfo;
-			gchar          *os;
 
 			from = lm_message_node_get_attribute (m->node, "from");
 			id = lm_message_node_get_attribute (m->node, "id");
@@ -1042,12 +1070,7 @@ app_iq_handler (LmMessageHandler *handler,
 			lm_message_node_add_child (node, "version", VERSION);
 			
 			uname (&osinfo);
-			os = g_strdup_printf ("%s %s %s",
-					      osinfo.sysname,
-					      osinfo.release,
-					      osinfo.machine);
-			lm_message_node_add_child (node, "os", os);
-			g_free (os);
+			lm_message_node_add_child (node, "os", osinfo.sysname);
 
 			lm_connection_send (connection, v, NULL);
 			lm_message_unref (v);
@@ -1080,7 +1103,7 @@ app_connection_open_cb (LmConnection *connection,
 	}
 
 	if (!priv->account->password || !priv->account->password[0]) {
-		password = gossip_password_dialog_run (GTK_WINDOW (priv->window));
+		password = gossip_password_dialog_run (priv->account, GTK_WINDOW (priv->window));
 		if (!password) {
 			app_disconnect (app);
 			return;
@@ -1110,22 +1133,19 @@ app_connection_open_cb (LmConnection *connection,
 }
 
 static void
-app_client_disconnected_cb (LmConnection *connection,
+app_client_disconnected_cb (LmConnection       *connection,
 			    LmDisconnectReason  reason, 
-			    GossipApp    *app)
+			    GossipApp          *app)
 {
 	GossipAppPriv *priv;
 	GtkWidget     *dialog;
-	GtkWidget     *menu, *item;
-	GossipStatus   old_status;
+	GossipStatus   status;
 
 	priv = app->priv;
 
 	/* Keep the old status if we are going to reconnect. */
-	menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (priv->option_menu));
-	item = gtk_menu_get_active (GTK_MENU (menu));
-	old_status = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "data"));
-
+	status = gossip_status_menu_get_status (priv->option_menu);
+	
 	app_set_status_indicator (app, GOSSIP_STATUS_OFFLINE);
 
 	g_signal_emit (app, signals[DISCONNECTED], 0);
@@ -1139,7 +1159,9 @@ app_client_disconnected_cb (LmConnection *connection,
 						   "Do you want to reconnect?"));
 		
 		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES) {
-			app_set_status (app, old_status);
+			app_set_status (app, status);
+		} else {
+			app_set_status (app, GOSSIP_STATUS_OFFLINE);
 		}
 	
 		gtk_widget_destroy (dialog);
@@ -1163,6 +1185,72 @@ gossip_app_join_group_chat (GossipApp   *app,
 	app_get_group_chat (app, room, server, nick, TRUE);
 }
 
+static gboolean
+app_idle_check_cb (GossipApp *app)
+{
+	GossipStatus   status;
+	GossipAppPriv *priv;
+	gint           idle;
+	gint           auto_away;
+	gint           auto_ext_away;
+	gboolean       is_away;
+
+	priv = app->priv;
+
+	if (!gconf_client_get_bool (gconf_client,
+				    "/apps/gossip/auto_away/enabled",
+				    NULL)) {
+		return TRUE;
+	}
+
+	status = gossip_status_menu_get_status (priv->option_menu);
+
+	switch (status) {
+	case GOSSIP_STATUS_OFFLINE:
+		return TRUE;
+
+	case GOSSIP_STATUS_AWAY:
+	case GOSSIP_STATUS_EXT_AWAY:
+		is_away = TRUE;
+		break;
+
+	default:
+		is_away = FALSE;
+		break;
+	}
+
+	auto_away = 60 * gconf_client_get_int (gconf_client,
+					       "/apps/gossip/auto_away/idle_time",
+					       NULL);
+	auto_ext_away = 60 * gconf_client_get_int (gconf_client,
+						   "/apps/gossip/auto_away/extended_idle_time",
+						   NULL);
+	
+	idle = gossip_idle_get_seconds ();
+
+	if (!is_away && idle > (auto_away)) {
+		d(g_print ("Going auto-away...\n"));
+
+		app_set_status (app, GOSSIP_STATUS_AWAY); /*, "Autoaway");*/
+		/*gossip_app_statusbar_push_notification (app, "Going auto-away");*/
+		priv->is_auto_away = TRUE;
+	}
+	else if (priv->is_auto_away &&
+		 status != GOSSIP_STATUS_EXT_AWAY && idle > auto_ext_away) {
+		d(g_print ("Going extended auto-away...\n"));
+		app_set_status (app, GOSSIP_STATUS_EXT_AWAY); /*, "Auto-Away (Ext)");*/
+		/*gossip_app_statusbar_push_notification (app, "Going extended auto-away");*/
+	}
+	else if (priv->is_auto_away && idle < auto_away) {
+		d(g_print ("Returning from auto-away...\n"));
+		app_set_status (app, GOSSIP_STATUS_AVAILABLE); /*, "Back from idle");*/
+		/*gossip_app_statusbar_push_notification (app, "Returning from auto-away");*/
+		priv->is_auto_away = FALSE;
+	}
+
+	return TRUE;
+}
+
 void
 gossip_app_connect (GossipAccount *account)
 {
@@ -1189,6 +1277,7 @@ gossip_app_connect (GossipAccount *account)
 
 	lm_connection_set_server (priv->connection, account->server);
 	lm_connection_set_port (priv->connection, account->port);
+	lm_connection_set_use_ssl (priv->connection, account->use_ssl);
 	
 	lm_connection_open (priv->connection,
 			    (LmResultFunction) app_connection_open_cb,
@@ -1239,17 +1328,6 @@ app_set_status (GossipApp *app, GossipStatus status)
 
 	d(g_print ("app_set_status\n"));
 
-	switch (status) {
-	case GOSSIP_STATUS_OFFLINE:
-	case GOSSIP_STATUS_EXT_AWAY:
-	case GOSSIP_STATUS_AWAY:
-	case GOSSIP_STATUS_BUSY:
-		gossip_sound_set_silent (TRUE);
-		break;
-	default:
-		gossip_sound_set_silent (FALSE);
-	}
-	
 	/* Disconnect. */
 	if (status == GOSSIP_STATUS_OFFLINE) {
 		if (lm_connection_is_open (priv->connection)) {
@@ -1449,6 +1527,7 @@ app_create_connection (GossipApp *app)
 
 	priv->connection = lm_connection_new (priv->account->server);
 	lm_connection_set_port (priv->connection, priv->account->port);
+	lm_connection_set_use_ssl (priv->connection, priv->account->use_ssl);
 
 	handler = lm_message_handler_new ((LmHandleMessageFunction) app_message_handler, 
 					  app, NULL);
@@ -1669,4 +1748,8 @@ app_complete_jid_to_string (gpointer data)
 	return (gchar *) gossip_jid_get_without_resource (jid);
 }
 
-
+GossipStatus 
+gossip_app_get_status (void)
+{
+	return gossip_status_menu_get_status (app->priv->option_menu);
+}
